<?php declare(strict_types=1);

namespace massivedynamic\lucidity;

use Mustache_Engine;
use Mustache_Loader_FilesystemLoader;
use Mustache_Loader_CascadingLoader;

class TemplateEngine extends Mustache_Engine {
	public function __construct(private Application $app, private string $templatesPath, string $partialsPath, string $cachePath) {
		parent::__construct([
			"cache" => $cachePath,
			"loader" => new Mustache_Loader_CascadingLoader([
			new Mustache_Loader_FilesystemLoader($templatesPath),
				new Mustache_Loader_FilesystemLoader(dirname(__FILE__)."/../views")
			]),
			"partials_loader" => new Mustache_Loader_CascadingLoader([
				new Mustache_Loader_FilesystemLoader($partialsPath),
				new Mustache_Loader_FilesystemLoader(dirname(__FILE__)."/../views/partials")
			]),
			"helpers" => [
				"notEmpty" => fn(array $array) : bool => count($array) > 0
			],
			"pragmas" => [Mustache_Engine::PRAGMA_BLOCKS, Mustache_Engine::PRAGMA_FILTERS]
		]);
	}

	public function view(string $template, array $context = []) : void {
		echo $this->render("$template.mustache", array_merge($this->app->globals(), $context));
	}
}
