<?php declare(strict_types=1);

namespace massivedynamic\lucidity\containers;

use Exception;
use ReflectionClass;

use massivedynamic\lucidity\Application;
use Psr\Container\ContainerInterface;

abstract class Container implements ContainerInterface {
	protected array $instances;

	public function __construct(protected Application $app) {
		$this->instances = [];
	}

	public function get(string $id) : mixed {
		if ($id === Application::class || is_subclass_of($id, Application::class))
			return $this->app;
		if (!$this->has($id))
			$this->instantiate($id);

		return $this->instances[$id];
	}

	public function has(string $id) : bool {
		return array_key_exists($id, $this->instances);
	}

	public function emplace(string $type, ...$args) : bool {
		if (array_key_exists($type, $this->instances))
			return false;
		$this->instances[$type] = new $type(...$args);

		return true;
	}

	protected function instantiate(string $id) : void {
		$classInfo   = new ReflectionClass($id);
		$constructor = $classInfo->getConstructor();
		$arguments   = [];

		if ($constructor === null) {
			$this->instances[$id] = $classInfo->newInstance();
			return;
		}

		foreach ($constructor->getParameters() as $parameter) {
			$type = $parameter->getType();
			if ($type === null)
				throw new Exception("constructor arguments need to name a type");
			array_push($arguments, $this->get($type->getName()));
		}
		$this->instances[$id] = $classInfo->newInstanceArgs($arguments);
	}
}
