<?php declare(strict_types=1);

namespace massivedynamic\lucidity\models;

use ReflectionClass;
use ReflectionProperty;
use ReflectionUnionType;

use massivedynamic\lucidity\Application;
use massivedynamic\lucidity\Utils;
use massivedynamic\lucidity\models\Model;
use massivedynamic\lucidity\services\Database;
use massivedynamic\lucidity\services\Service;

abstract class DatabaseModel extends Model {
	protected static ?string $TableName = null;

	protected Service | Database $db;

	public function __construct() {
		$this->db = Application::Instance()->fetchService(Database::class);
	}

	public function save()  {
		$classInfo    = new ReflectionClass($this);
		$properties   = $classInfo->getProperties(ReflectionProperty::IS_PROTECTED);
		$columns      = [];
		$values       = [];
		$types        = [];

		foreach ($properties as $property) {
			if ($property->getType() instanceof ReflectionUnionType || $property->isStatic() || !$property->getType()->isBuiltin())
				continue;

			array_push($columns, $property->name);
			array_push($values, $this->{$property->name});
			array_push($types, Database::PrimitiveTypeToBindParam($property->getType()->getName()));
		}
		$placeholders = array_fill(0, count($columns), '?');

		$qs = "INSERT INTO ".self::TableName()." (".implode(", ", $columns).") VALUES (".implode(", ", $placeholders).")";
		$query = $this->db->prepare($qs);
		$query->bind_param(implode("", $types), ...$values);
	}

	public function all() : array {
		$result = $this->db->query("SELECT * FROM ".self::TableName());

		$out = [];
		while ($row = $result->fetch_object())
			array_push($out, $row);

		return $out;
	}

	private static function TableName() : string {
		return self::$TableName ?? self::Type2TableName(static::class);
	}

	private static function Type2TableName(string $type) : string {
		$parts = explode("\\", $type);
		$basename = strtolower(array_pop($parts));

		return substr($basename, -1) === "s" ? $basename : $basename."s";
	}
}
