<?php declare(strict_types=1);

namespace massivedynamic\lucidity\validators;

use Exception;

use massivedynamic\lucidity\exceptions\UnknownValidatorException;
use massivedynamic\lucidity\validators\Validation;
use massivedynamic\lucidity\validators\ValidationType;
use massivedynamic\lucidity\Utils;

class Validator {
	private array $errors;
	private array $validators;

	public function __construct(private array $pool, array $validation) {
		$this->errors     = [];
		$this->validators = self::ParseValidationArray($validation);
	}

	public function validate() : ?array {
		foreach ($this->validators as $key => $validators) {
			foreach($validators as $validator) {
				switch ($validator->type) {
					case ValidationType::REQUIRED:
						$this->validateRequired($key);
						break;
					case ValidationType::MIN:
						$this->validateMin($key, $validator);
						break;
					case ValidationType::MAX:
						$this->validateMax($key, $validator);
						break;
				}
			}
		}
		return count($this->errors) > 0 ? $this->errors : null;
	}

	private function validateMin(string $key, Validation $validator) : void {
		$length = strlen($this->pool[$key]);
		if ($length < intval($validator->parameter))
			array_push($this->errors, "Field '$key' must be at least $validator->parameter characters long (is $length)");

	}

	private function validateMax(string $key, Validation $validator) : void {
		$length = strlen($this->pool[$key]);
		if ($length > intval($validator->parameter))
			array_push($this->errors, "Field '$key' must be at least $validator->parameter characters long (is $length)");

	}

	private function validateRequired(string $key) : void {
		if (!isset($this->pool[$key]))
			array_push($this->errors, "Field '$key' is required");
	}

	/**
	 * @@param array<string, array> $validation
	 */
	public static function ParseValidationArray(array $validation) : array {
		$out = [];

		foreach ($validation as $key => $validators) {
			$validators = explode("|", $validators);
			if (count($validators) < 1)
				continue;

			$out[$key] = [];
			foreach ($validators as $validator) {
				$parts             = explode(":", $validator);
				[$type, $argument] = count($parts) === 2 ? $parts : [$parts[0], ""];
				$argument          = $argument ?? "";
				$typeResolved      = ValidationType::tryFrom(strtolower($type));

				if ($typeResolved === null)
					throw new UnknownValidatorException($type);

				$validator = new Validation($typeResolved, $argument);
				array_push($out[$key], $validator);
			}
		}

		return $out;
	}
}
